#include <stdio.h>
#include <jpeglib.h>
#include <assert.h>
#include "libjpegrest.h"


/* Prototypes */
static int scanlines_to_matrix(struct jpeg_decompress_struct * srcinfo,
		struct bitmap *bmap);

void bitmap_jpeg_init(int *argn, char **argv)
{
	return;
}

int
bitmap_filename_jpeg_get(const char *filename, struct bitmap *bmap,
	enum jpeg_conditions conds)
{
	FILE *input;
	int res;

	/* Opening the FILE */
	input = fopen(filename,"r");
	if (input == NULL)
	{
		fprintf(stderr,"The file \"%s\" cannot be opened.\n",
			filename);
		res = -1;
	} else
	{
		res = bitmap_stdio_jpeg_get(input, bmap, conds);
		fclose(input);
	}
	
	return res;
}

int
bitmap_stdio_jpeg_get(FILE *input, struct bitmap *bmap,
	enum jpeg_conditions conds)
{
	struct jpeg_decompress_struct srcinfo;
	struct jpeg_error_mgr jsrcerr;

	srcinfo.err = jpeg_std_error(&jsrcerr);

	jpeg_create_decompress(&srcinfo);

	jpeg_stdio_src(&srcinfo, input);
	jpeg_read_header(&srcinfo, TRUE);

	
	switch(conds)
	{
	case READ_RGB:
		srcinfo.out_color_space = JCS_RGB;
		scanlines_to_matrix(&srcinfo, bmap);
		break;
	case READ_YCbCr:
		srcinfo.out_color_space = JCS_YCbCr;
		scanlines_to_matrix(&srcinfo, bmap);
		break;
	case READ_PLANES:
		srcinfo.out_color_space = srcinfo.jpeg_color_space;
		scanlines_to_matrix(&srcinfo, bmap);
	default:
		fprintf(stderr, "JPEG Bitmap open method not supported.\n");
	}

	jpeg_finish_decompress(&srcinfo);
	jpeg_destroy_decompress(&srcinfo);
	return 0;
}

static int
scanlines_to_matrix(struct jpeg_decompress_struct * srcinfo,
		struct bitmap *bmap)
{
	int row, column, plane;
	JSAMPARRAY scanlines; /* buffer of scanlines */
	int nscanlines; /* number of scanlines in the buffer */
	int isl;  /* iterator over buffer scanlines */
	int read_scanlines; /* number of scanlines read at once */

	/* Set our JPEG decompression parameters */
	srcinfo->do_fancy_upsampling = 1;
	srcinfo->dct_method = JDCT_FLOAT;

	/* Get the number of planes */
	if (srcinfo->num_components == 1 &&
			srcinfo->jpeg_color_space == JCS_GRAYSCALE) {
		srcinfo->out_color_components = 1;
		srcinfo->out_color_space = JCS_GRAYSCALE;
	}
	else {
		srcinfo->out_color_components = 3;
		srcinfo->out_color_space = JCS_RGB;
	}

	/* Start decompression of pixels */
	jpeg_start_decompress(srcinfo);


	*bmap = bitmap_alloc(srcinfo->out_color_components,
			srcinfo->output_width,
			srcinfo->output_height);
	/* Get the recommended number of buffer scanlines */
	nscanlines = srcinfo->rec_outbuf_height;

	/* Get memory for the buffer scanline pointers */
	scanlines = malloc(sizeof(JSAMPROW) * nscanlines);
	/* Get memory for each scanline in the buffer */
	for (isl = 0; isl < nscanlines; ++isl)
	{
		scanlines[isl] = malloc(sizeof(JSAMPLE)
				* srcinfo->output_width
				* srcinfo->out_color_components);
	}

	row = 0;
	while (srcinfo->output_scanline < srcinfo->output_height) {
		read_scanlines =
			jpeg_read_scanlines(srcinfo, scanlines, nscanlines);
		for (isl = 0; isl < read_scanlines; ++isl)
		{
			for (column = 0; column < srcinfo->output_width;
					++column) {
				for (plane = 0; plane < bmap->nplanes; plane++)
					bmap->m[plane]->data[row *
						bmap->m[plane]->tda + column] =
						scanlines[isl][column *
						bmap->nplanes + plane];
			}
			++row;
		}
	}
	
	for (isl = 0; isl < nscanlines; ++isl)
		free(scanlines[isl]);
	free(scanlines);
	bmap->is_ready = 1;

	return row;
}
